
--[[--------------------------------------------------------------------
	Auc-Stat-TheUndermineJournal - The Undermine Journal price statistics module
	Copyright (c) 2011, 2012 Johnny C. Lam
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
	TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
--]]--------------------------------------------------------------------

--[[
	Development was resumed by Heavy Stone Crab in 2015.
--]]

if not AucAdvanced then return end

local libName, libType = "The Undermine Journal", "Stat"
local lib, parent, private = AucAdvanced.NewModule(libType, libName)

if not lib then return end

local aucPrint, decode, _, _, replicate, empty, get, set, default, debugPrint, fill, _TRANS = AucAdvanced.GetModuleLocals()
local bellCurve = AucAdvanced.API.GenerateBellCurve()
local wipe = wipe
local Resources = AucAdvanced.Resources
local ResolveServerKey = AucAdvanced.ResolveServerKey
local r1, g1, b1 = 0.4, 1.0, 0.4 -- 102, 255, 102
local r2, g2, b2 = 0.9, 0.8, 0.5 -- TUJ

-- TUJ data table for market data from most-recently requested item.
local tujData = { }

local priceArray = { }

lib.Processors = {
	config = function(callbackType, ...)
			private.SetupConfigGui(...)
		end,
	load = function(callbackType, addon)
			if private.OnLoad then
				private.OnLoad(addon)
			end
		end,
	itemtooltip = function(callbackType, ...)
			private.ProcessTooltip(...)
		end,
	battlepettooltip = function(callbackType, ...)
			private.ProcessTooltip(...)
		end
}

-- lib.GetPrice()
--     (optional) Returns the estimated price for an item link.
function lib.GetPrice(hyperlink, serverKey)
	if not get("stat.TUJ.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end
	
	return tujData.recent, tujData.market, tujData.stddev, tujData.globalMedian, tujData.globalMean,
		tujData.globalStdDev, tujData.days, tujData.age
end

-- lib.GetPriceColumns()
--     (optional) Returns the column names for GetPrice.
function lib.GetPriceColumns()
	return "3-Day Price", "14-Day Price", "Std Dev", "Global Median Price", "Global Mean Price",
		"Global Std Dev", "Last Seen", "Data Age"
end

-- lib.GetPriceArray()
--     Returns pricing and other statistical info in an array.
function lib.GetPriceArray(hyperlink, serverKey)
	if not get("stat.TUJ.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end
	
	wipe(priceArray)
	
	local pricing = math.huge
	
	if tujData.recent and get("stat.TUJ.includeRecent") then
		if tujData.recent < pricing then
			pricing = tujData.recent
		end
	end
	
	if tujData.market and get("stat.TUJ.includeMarket") then
		if tujData.market < pricing then
			pricing = tujData.market
		end
	end
	
	if tujData.globalMedian and get("stat.TUJ.includeGlobalMedian") then
		if tujData.globalMedian < pricing then
			pricing = tujData.globalMedian
		end
	end
	
	if tujData.globalMean and get("stat.TUJ.includeGlobalMean") then
		if tujData.globalMean < pricing then
			pricing = tujData.globalMean
		end
	end
	
	if tujData.stdDev and tujData.market and get("stat.TUJ.includeStdDev") then
		if tujData.stddev > (tujData.market * (get("stat.TUJ.thresholdStdDev") / 100)) then
			pricing = 0
		end
	end
	
	if tujData.globalStdDev and tujData.globalMedian and get("stat.TUJ.includeGlobalStdDev") then
		if tujData.globalStdDev > (tujData.globalMedian * (get("stat.TUJ.thresholdGlobalStdDev") / 100)) then
			pricing = 0
		end
	end
	
	if pricing == math.huge then
		pricing = 0
	end
	
	priceArray.price = pricing
	priceArray.seen = 0
	
	-- Lots of optional values; not entirely sure what modules use which values.
	--[[
	priceArray.latest = 0
	priceArray.market = 0
	priceArray.median = 0
	priceArray.stddev = 0
	priceArray.cstddev = 0
	priceArray.region_median = 0
	priceArray.region_stddev = 0
	priceArray.region_price = 0
	priceArray.region_cstddev = 0
	priceArray.qty = 0
	--]]
	
	return priceArray
end

-- lib.GetItemPDF()
--     Returns the Probability Density Function for an item link.
function lib.GetItemPDF(hyperlink, serverKey)
	if not get("stat.TUJ.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end
	
	local mean, stddev = tujData.globalMean, tujData.globalStdDev
	
	if not mean or not stddev or stddev == 0 then
		-- No available data.
		return
	end
	
	-- Calculate the lower and upper bounds as +/- 3 standard deviations.
	local lower, upper = (mean - 3 * stddev), (mean + 3 * stddev)

	bellCurve:SetParameters(mean, stddev)
	
	return bellCurve, lower, upper
end

function private.OnLoad(addon)
	if addon ~= "auc-stat-theunderminejournal" then return end
	
	default("stat.TUJ.enable", true)
	default("stat.TUJ.includeRecent", false)
	default("stat.TUJ.includeMarket", false)
	default("stat.TUJ.includeStdDev", false)
	default("stat.TUJ.thresholdStdDev", 50)
	default("stat.TUJ.includeGlobalMedian", true)
	default("stat.TUJ.includeGlobalMean", false)
	default("stat.TUJ.includeGlobalStdDev", false)
	default("stat.TUJ.thresholdGlobalStdDev", 50)
	default("stat.TUJ.tooltip", true)
	default("stat.TUJ.disableOriginalTooltip", true)
	default("stat.TUJ.multiplyStackSize", false)
	--[[
	default("stat.TUJ.input", true)
	default("stat.TUJ.itemid", true)
	default("stat.TUJ.bonuses", true)
	default("stat.TUJ.age", true)
	default("stat.TUJ.species", true)
	default("stat.TUJ.breed", true)
	default("stat.TUJ.quality", true)
	]]--
	default("stat.TUJ.recent", true)
	default("stat.TUJ.market", true)
	default("stat.TUJ.stddev", true)
	default("stat.TUJ.globalMedian", true)
	default("stat.TUJ.globalMean", true)
	default("stat.TUJ.globalStdDev", true)
	default("stat.TUJ.days", true)
	
	-- Only run this function once.
	private.OnLoad = nil
end

-- private.GetInfo(hyperlink, serverKey)
--     Returns the market info for the requested item in the tujData table.
function private.GetInfo(hyperlink, serverKey)
	local serverKey = ResolveServerKey(serverKey)
	
	if serverKey ~= Resources.ServerKey then return end
	
	local linkType, itemId, suffix, factor = decode(hyperlink)
	
	if linkType ~= "item" and linkType ~= "battlepet" then return nil end
	
	wipe(tujData)
	
	if TUJMarketInfo then
		TUJMarketInfo(hyperlink, tujData)
	end
	
	if not (tujData.itemid or tujData.breed) then return nil end
	
	return itemId or tujData.breed
end

function private.SetupConfigGui(gui)
	local id = gui:AddTab(lib.libName, lib.libType .. " Modules")
	
	gui:AddHelp(id, "what TUJ",
		_TRANS("TUJ_Help_Question1"),
		_TRANS("TUJ_Help_Answer1")
	)
	gui:AddHelp(id, "what auc stat TUJ",
		_TRANS("TUJ_Help_Question2"),
		_TRANS("TUJ_Help_Answer2")
	)
	gui:AddHelp(id, "which setup",
		_TRANS("TUJ_Help_Question3"),
		_TRANS("TUJ_Help_Answer3")
	)
	gui:AddHelp(id, "what setup",
		_TRANS("TUJ_Help_Question4"),
		_TRANS("TUJ_Help_Answer4")
	)
	
	-- All options here will be duplicated in the tooltip frame.
	function private.addTooltipControls(id)
		gui:MakeScrollable(id)
		
		gui:AddControl(id, "Header",     0,    _TRANS("TUJ_Interface_Options"))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
		
		gui:AddControl(id, "Checkbox",   0, 1, "stat.TUJ.disableOriginalTooltip", _TRANS("TUJ_Interface_DisableOriginalTooltip"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_DisableOriginalTooltip"))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
		
		gui:AddControl(id, "Checkbox",   0, 1, "stat.TUJ.enable", _TRANS("TUJ_Interface_Enable"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_Enable"))
		
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeRecent", _TRANS("TUJ_Interface_IncludeRecent"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeRecent"))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeMarket", _TRANS("TUJ_Interface_IncludeMarket"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeMarket"))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeStdDev", _TRANS("TUJ_Interface_IncludeStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeStdDev"))
		gui:AddControl(id, "WideSlider", 0, 6, "stat.TUJ.thresholdStdDev", 0, 150, 1, _TRANS("TUJ_Interface_ThresholdStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ThresholdStdDev"))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeGlobalMedian", _TRANS("TUJ_Interface_IncludeGlobalMedian"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeGlobalMedian"))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeGlobalMean", _TRANS("TUJ_Interface_IncludeGlobalMean"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeGlobalMean"))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.includeGlobalStdDev", _TRANS("TUJ_Interface_IncludeGlobalStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_IncludeGlobalStdDev"))
		gui:AddControl(id, "WideSlider", 0, 6, "stat.TUJ.thresholdGlobalStdDev", 0, 150, 1, _TRANS("TUJ_Interface_ThresholdGlobalStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ThresholdGlobalStdDev"))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
		
		gui:AddControl(id, "Checkbox",   0, 4, "stat.TUJ.tooltip", _TRANS("TUJ_Interface_ToggleTooltip"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleTooltip"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.multiplyStackSize", _TRANS("TUJ_Interface_ToggleMultiplyStackSize"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleMultiplyStackSize"))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.recent", _TRANS("TUJ_Interface_ToggleRecent"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleRecent"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.market", _TRANS("TUJ_Interface_ToggleMarket"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleMarket"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.stddev", _TRANS("TUJ_Interface_ToggleStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleStdDev"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.globalMedian", _TRANS("TUJ_Interface_ToggleGlobalMedian"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleGlobalMedian"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.globalMean", _TRANS("TUJ_Interface_ToggleGlobalMean"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleGlobalMean"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.globalStdDev", _TRANS("TUJ_Interface_ToggleGlobalStdDev"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleGlobalStdDev"))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.TUJ.days", _TRANS("TUJ_Interface_ToggleDays"))
		gui:AddTip(id, _TRANS("TUJ_HelpTooltip_ToggleDays"))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
	end
	
	local tooltipID = AucAdvanced.Settings.Gui.tooltipID
	
	private.addTooltipControls(id)
	
	if tooltipID then
		private.addTooltipControls(tooltipID)
	end
end

function private.ProcessTooltip(tooltip, hyperlink, serverKey, quantity, decoded, additional, order)
	if TUJTooltip then
		if get("stat.TUJ.disableOriginalTooltip") then
			TUJTooltip(false)
		else
			TUJTooltip(true)
		end
	end
	
	if not (get("stat.TUJ.enable") and get("stat.TUJ.tooltip")) then return end
	if not private.GetInfo(hyperlink, nil) then return end
	
	if not quantity or quantity < 1 then
		quantity = 1
	end
	
	if get("stat.TUJ.multiplyStackSize") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_StackSize"):format(quantity), r1, g1, b1)
	else
		quantity = 1
	end
	
	if tujData.age then
		if tujData.age > (3 * 24 * 60 * 60) then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_Age"):format(tujData.age / 24 / 60 / 60, (tujData.age / 24 / 60 / 60 - math.floor(tujData.age / 24 / 60 / 60)) * 24), r1, g1, b1)
		end
	end
	
	if tujData.recent and get("stat.TUJ.recent") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_Recent"), tujData.recent * quantity)
	end
	
	if tujData.market and get("stat.TUJ.market") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_Market"), tujData.market * quantity)
	end
	
	if tujData.stddev and get("stat.TUJ.stddev") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_StdDev"), tujData.stddev * quantity, r2, g2, b2)
	end
	
	if tujData.globalMedian and get("stat.TUJ.globalMedian") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_GlobalMedian"), tujData.globalMedian * quantity)
	end
	
	if tujData.globalMean and get("stat.TUJ.globalMean") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_GlobalMean"), tujData.globalMean * quantity)
	end
	
	if tujData.globalStdDev and get("stat.TUJ.globalStdDev") then
		tooltip:AddLine(_TRANS("TUJ_Tooltip_GlobalStdDev"), tujData.globalStdDev * quantity, r2, g2, b2)
	end
	
	if tujData.days and get("stat.TUJ.days") then
		if tujData.days == 255 then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_NotSeen"), r1, g1, b1)
		elseif tujData.days == 252 then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_Vendors"), r1, g1, b1)
		elseif tujData.days > 250 then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_LastSeen250"), r1, g1, b1)
		elseif tujData.days > 1 then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_Days"):format(tujData.days), r1, g1, b1)
		elseif tujData.days == 1 then
			tooltip:AddLine(_TRANS("TUJ_Tooltip_Day"), r1, g1, b1)
		end
	end
end
